@@ -3,13 +3,6 @@ package iTunes::Sid;
 use strict;
 use warnings;
 
-use Carp qw( carp croak );
-use List::Util qw( first );
-use if $^O ne "MSWin32", 'Parse::DMIDecode';
-use if $^O ne "MSWin32", 'Sys::Info';
-use if $^O ne "MSWin32", 'vars', qw( $Registry );
-use if $^O eq 'MSWin32', 'Win32::TieRegistry';
-use if $^O eq 'MSWin32', 'Win32::DriveInfo';
 use File::Path;
 use Digest::MD5;
 use File::Basename;
@@ -17,22 +10,36 @@ use Crypt::Rijndael;
 use File::Find::Rule;
 use Crypt::AppleTwoFish;
 use Parse::Win32Registry;
+use Carp qw( carp croak );
+use List::Util qw( first );
 use Scalar::Util qw( weaken );
 
 use iTunes::Sid::Atom;
 
-our $VERSION = '0.033';
+our $VERSION = '0.40';
+
+#-----------  determines whether data is saved to .drms by default -----------#
+
+my $auto_save_guid_and_mac_in_drms_dir = 1;
+my $auto_read_guid_and_mac_in_drms_dir = 1;
+
+#--------------- package defaults for searches ----------------#
+
+# this is often wrong, but may sugggest Linux defaults for others
+# to adapt to a typical dual boot setup, may need to substitute
+#     'disk-2' for 'disk-1' or '/mnt' for '/media', etc.
+my @default_SCInfo_directory = (
+    '/media/disk-1/ProgramData/Apple Computer/iTunes/SC Info/',
+    '/media/disk-1/Documents and Settings/Application Data/Apple Computer/iTunes/SC Info',
+    '/ProgramData/Apple Computer/iTunes/SC Info/',
+    '/Documents and Settings/All Users/Application Data/Apple Computer/iTunes/SC Info',    
+);
 
 #------------------- useful constants -----------------------------#
 
 my $sid_version_prefix_size = 12;
 my %sid_version_ok = ( 0x60002 => 1 );
 
-#--------------- package defaults for searches ----------------#
-
-my $default_SCInfo_directory = '/media/disk-1/Windows/System32/config/';
-my $default_hive_file        = './_REGISTRY_MACHINE_SOFTWARE';
-
 #-------------- documented public functions ------------------------#
 
 sub check_encrypted_sid {
@@ -47,19 +54,19 @@ sub _random_bytes {
     my $str = '';
     while ( 1 .. $num_wanted ) {
         $str .= chr rand(255);
-        return $str;
     }
+    return $str;
+}
+
+sub _decode_16_bytes {
+    my ( $bytes, $key ) = @_;
+    carp("wrong key length") unless length $bytes == 16;
+    my $alg = Crypt::Rijndael->new( $key, Crypt::Rijndael::MODE_CBC );
+    return $alg->decrypt($bytes);
 }
 
 #--- functions that will likely change with new iTunes releases ----#
 
-sub _drms_directory {
-    my $home_dir = $ENV{APPDATA} || $ENV{HOME} || q{~};
-    my $sPfix = ( $^O eq 'MSWin32' ) ? q{} : q{.};
-    my $dirSep = q{/};
-    return $home_dir . $dirSep . $sPfix . 'drms' . $dirSep;
-}
-
 sub get_sid_iv {
 
     # iTunes 7 default ???
@@ -84,10 +91,11 @@ sub get_sid_key {
     return $key;
 }
 
-sub _trim_head_nulls {
-    my ($rbuf) = @_;
-    $$rbuf =~ s/^(\x00\x00\x00\x00)+//;
-    return $rbuf;
+sub _drms_directory {
+    my $home_dir = $ENV{APPDATA} || $ENV{HOME} || q{~};
+    my $sPfix = ( $^O eq 'MSWin32' ) ? q{} : q{.};
+    my $dirSep = q{/};
+    return $home_dir . $dirSep . $sPfix . 'drms' . $dirSep;
 }
 
 sub _info_MD5 {
@@ -101,115 +109,60 @@ sub _info_MD5 {
     return substr $digst, 0, 4;
 }
 
-sub _registry_info_MD5 {
-    my ( $registry_key, $withnull ) = @_;
-    my $val = $Registry->{$registry_key};
-    if ($withnull) {
-        $val .= "\x00";
-    }
-    my $md5 = Digest::MD5->new;
-    $md5->add($val);
-    my $digst = $md5->digest;
-    return substr $digst, 0, 4;
-}
-
 sub _find_sid {
     my ($sid) = @_;
 
     # check the default dir if there is one
-    if ($default_SCInfo_directory) {
-        my $use_last_dir = $default_SCInfo_directory . '/' . $sid;
-        return $use_last_dir if -e $use_last_dir;
+    for my $scdir  (@default_SCInfo_directory) {
+        my $default_pathname = $scdir . '/' . $sid;
+       if ( -f $default_pathname ) {
+            return $default_pathname ;
+        }
     }
 
-    my $darwin_sid = "/Users/Shared/SC Info/$sid";
-    my $windows_partial_XP_sid =
-      "/Application Data/Apple Computer/iTunes/SC Info/$sid";
-    my $windows_partial_Vista_sid = "/Apple Computer/iTunes/SC Info/$sid";
-    my $windows_xp_sid    = $ENV{ALLUSERSPROFILE} . $windows_partial_XP_sid;
-    my $windows_vista_sid = $ENV{ALLUSERSPROFILE} . $windows_partial_Vista_sid;
+    my $darwin_sid_dir = "/Users/Shared/SC Info/";
+    my $darwin_sid     = $darwin_sid_dir . $sid;
 
-    return $darwin_sid        if -f $darwin_sid;
-    return $windows_xp_sid    if -f $windows_xp_sid;
-    return $windows_vista_sid if -f $windows_vista_sid;
+    my $windows_sid_dir = "/Apple Computer/iTunes/SC Info/";
+    my $windows_sid     = $windows_sid_dir . $sid;
 
-    # do a find, hoping we have a Windows or Darwin drive mounted for find
-    return unless $sid =~ m/([\w\d\-\_\.\s]+)/;    # taint elimination
-    $sid = $1;
-    my @lines = `find / -name $sid`;
-    my @results = grep { m|$darwin_sid| } @lines;
-    if ( scalar @results ) {
-        $default_SCInfo_directory = dirname( $results[0] );
-        return $results[0];
-    }
-    @results =
-      grep { m/$windows_partial_XP_sid|$windows_partial_Vista_sid/ } @lines;
-    if ( scalar @results ) {
-        $default_SCInfo_directory = dirname( $results[0] );
-        return $results[0];
+    my $dir_filter = qr{$darwin_sid_dir|$windows_sid_dir};
+    my @sid_dirs =
+      File::Find::Rule->directory()->maxdepth(8)->name($dir_filter)->in('/');
+    my @sids = File::Find::Rule->file()->name($sid)->in(@sid_dirs);
+    if ( scalar @sids ) {
+        push @default_SCInfo_directory, dirname( $sids[0] );
+        return $sids[0];
     }
     return;
 }
 
-sub _get_hive_file {
-
-    # if there is a default file to use, return it
-    return $default_hive_file if -f $default_hive_file;
-
-    # Registry search for Windows product ID
-    # need a mounted Windows partition or drive in /media/ for this
-    my ( $mount_root_dir, $dir_filter, $file_filter );
-    if ( $^O eq 'MSWin32' ) {
-        $mount_root_dir = "\\";
-        $dir_filter     = qr{System Volume Information/_restore.+/snapshot};
-        $file_filter    = '_REGISTRY_MACHINE_SOFTWARE';
-    }
-    else {
-        $mount_root_dir = '/media';
-        $dir_filter     = 'config';
-        $file_filter    = 'SOFTWARE';
-    }
-    my $software_registry_file;
-    my @software_registry_dirs =
-      File::Find::Rule->directory()->maxdepth(4)->name($dir_filter)
-      ->in($mount_root_dir);
-    my @registry_files =
-      File::Find::Rule->file()->name($file_filter)->in(@software_registry_dirs);
-    if ( scalar @registry_files ) {
-        return $registry_files[0];
-    }
-    else {
-        carp("Cannot find the current Windows software registry file");
-        return;
-    }
+sub _trim_head_nulls {
+    my ($rbuf) = @_;
+    $$rbuf =~ s/^(\x00\x00\x00\x00)+//;
+    return $rbuf;
 }
 
-sub _decode_16_bytes {
-    my ( $bytes, $key ) = @_;
-    carp("wrong key length") unless length $bytes == 16;
-    my $alg = Crypt::Rijndael->new( $key, Crypt::Rijndael::MODE_CBC );
-    return $alg->decrypt($bytes);
-}
-
-no bigint;
-
 #----------------- class methods ------------------------------#
 
 sub new {
     my ( $class, %args ) = @_;
     my $self = {};
     bless( $self, $class );
-    foreach my $k (qw( DEBUG DEBUGDUMPFILE file key iv mac)) {
+    foreach my $k (qw( DEBUG DEBUGDUMPFILE file key iv mac 
+      regdata iTunes_platform scinfo_directory )) {
         $self->{$k} = $args{$k} if exists $args{$k};
     }
     $self->{meta} = {};
     $self->{DEBUG} = 0 unless exists $self->{DEBUG};
+    push @default_SCInfo_directory, $self->{scinfo_directory}
+      if exists $self->{scinfo_directory};
     if ( exists $self->{file} ) {
         if ( $self->{file} eq 'SCINFOSIDB' ) {
-            $self->{file} = _find_sid('SC INFO.sidb');
+            $self->{file} = _find_sid('SC Info.sidb');
         }
         elsif ( $self->{file} eq 'SCINFOSIDD' ) {
-            $self->{file} = _find_sid('SC INFO.sidd');
+            $self->{file} = _find_sid('SC Info.sidd');
         }
         $self->ReadFile( $self->{file}, $self->key(), $self->iv() );
         $self->ParseBuffer();
@@ -226,14 +179,16 @@ sub DESTROY {
 
 sub ReadFile {
     my ( $self, $infile, $key, $iv ) = @_;
-    open( my $infh, '<', $infile ) or croak("Cannot open input $infile: $!");
+    open( my $infh, '<', $infile )
+      or croak("Cannot open input $infile: $!");
     binmode $infh;
-    read( $infh, $self->{buffer}, -s $infile ) or croak("Bad file read: $!");
+    read( $infh, $self->{buffer}, -s $infile )
+      or croak("Bad file read: $!");
     close $infh;
     if ($key) {
         $self->key($key);
         $self->iv($iv);
-        $self->decrypt();
+        $self->decrypt() or croak "bad decrypt";
     }
     $self->{meta}->{filesize} = length $self->{buffer};
 }
@@ -279,7 +234,8 @@ sub WriteFileEncrypted {
 
 sub ParseSidContainer {
     my ( $self, $parent, $posit, $end_posit ) = @_;
-    my $pAtom = $parent->getNodeValue() or croak "Cannot get atom from node";
+    my $pAtom = $parent->getNodeValue()
+      or croak "Cannot get atom from node";
     $posit     = $pAtom->start + $pAtom->offset unless defined $posit;
     $end_posit = $pAtom->start + $pAtom->size   unless $end_posit;
     while ( $posit < $end_posit ) {
@@ -334,6 +290,47 @@ sub DumpTree {
     }
 }
 
+sub _save_mac_to_drms_dir {
+    my ($self)    = @_;
+    my $dest_file = _drms_directory . 'mac';
+    my $mac       = $self->mac;
+    if ($mac) {
+        if ( open my $fh, '>', $dest_file ) {
+            binmode $fh;
+            print "saving \"$mac\" to file $dest_file\n" if $self->{DEBUG};
+            print $fh "mac=$mac";
+            close $fh;
+        }
+        else {
+            carp("Cannot open $dest_file for writing: $!");
+        }
+    }
+    else {
+        carp("Cannot find MAC to write to file $dest_file");
+    }
+    return $mac;
+}
+
+sub _read_mac_from_drms_dir {
+    my ($self) = @_;
+    my $mac;
+    my $in_file = _drms_directory . 'mac';
+    if ( -f $in_file and open my $fh, '<', $in_file ) {
+        binmode $fh;
+        my $s = <$fh>;
+        close $fh;
+        if ( $s =~ m/^mac=(.+)/ ) {
+            $mac = $1;
+            $self->{mac} = $mac;
+            print "got mac from file $in_file\n" if $self->{DEBUG};
+        }
+        else {
+            carp("mac not readable: file first line should be: mac=.\{6\}");
+        }
+    }
+    return $mac;
+}
+
 sub FindAtom {
     my ( $self, $type ) = @_;
     return unless $self->{root};
@@ -348,7 +345,7 @@ sub FindAtom {
 sub key {
     my ( $self, $newkey ) = @_;
     $self->{key} = $newkey if defined $newkey;
-    $self->{mac} ||= $self->_get_hardware_data_string();
+#    $self->{mac} ||= $self->_get_hardware_data_string();
     if ( !$self->{key} or $self->{key} eq 'FIND' ) {
         $self->{key} = get_sid_key( $self->mac() );
     }
@@ -367,8 +364,19 @@ sub iv {
 sub mac {
     my ( $self, $newmac ) = @_;
     $self->{mac} = $newmac if defined $newmac;
-    if ( !$self->{iv} ) {
-        $self->{mac} = $self->_get_hardware_data_string();
+    if ( !$self->{mac} ) {
+        if($auto_read_guid_and_mac_in_drms_dir) {
+            $self->{mac} = $self->_read_mac_from_drms_dir();
+        }
+        if ( !$self->{mac} ) {
+            $self->{mac} = $self->_get_hardware_data_string();
+            if ( $self->{mac} and $auto_save_guid_and_mac_in_drms_dir ) {
+                $self->_save_mac_to_drms_dir( $self->{mac} );
+            }
+        }
+        if ( !$self->{mac} ) {
+            carp("Cannot locate mac / hardware ID string: $!");
+        }
     }
     return $self->{mac};
 }
@@ -402,13 +410,14 @@ sub decrypt {
     $alg->set_iv( $self->iv() );
     $buf = $alg->decrypt($buf);
     _trim_head_nulls( \$buf );
-    if ( iTunes::Sid::Atom::isRootAtomType( substr $buf, 4, 4 ) ) {
+    my $root_atom_type = substr $buf, 4, 4;
+    if ( iTunes::Sid::Atom::isRootAtomType( $root_atom_type) ) {
         $self->{buffer} = $buf;
         return $buf;
     }
     else {
         carp( "Bad aes128 sid decrypt with key ",
-            $self->key, " iv ", $self->iv );
+            $self->key, " iv ", $self->iv, " result $root_atom_type" );
         return;
     }
 }
@@ -424,12 +433,13 @@ sub sid_version {
 }
 
 sub _get_hardware_data_string {
-    my ( $self, $iTunes_platform, $mount_root_dir ) = @_;
+    my ($self) = @_;
     return $self->{mac} if $self->{mac};
-    $iTunes_platform ||= 'MSWin32';
-    my $hardware_string = "\x00\x00\x00\x00\x00\x00";
 
-    if ( $iTunes_platform eq "Darwin" ) {
+    my $hardware_string = "\x00\x00\x00\x00\x00\x00";
+    
+    my $platform = $self->{platform};
+    if ( $platform and $platform eq "Darwin" ) {
 
         # OS X was the iTunes platform that made the SC Info files we will use,
         # so the ethernet MAC address should work ok
@@ -446,95 +456,77 @@ sub _get_hardware_data_string {
             return;
         }
     }
-    elsif ( $^O eq "MSWin32" ) {
-        my $hw_id_md5 = Digest::MD5->new;
-        ( undef, my $drive_serial_number ) = Win32::DriveInfo::VolumeInfo('C');
-        $drive_serial_number =~ s/\W//;
-        $drive_serial_number = pack "L", hex($drive_serial_number);
-        $hw_id_md5->add("cache-controlEthernet");
-
-        my $md5 = Digest::MD5->new;
-        $md5->add( substr( $drive_serial_number, 0, 4 ) );
-        my $ds_digest = $md5->digest;
-        $hw_id_md5->add( substr $ds_digest, 0, 4 );
-        $hw_id_md5->add(
-            _registry_info_MD5(
-                "LMachine\\HARDWARE\\DESCRIPTION\\System\\SystemBiosVersion", 0
-            )
-        );
-        $hw_id_md5->add(
-            _registry_info_MD5(
-"LMachine\\HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\0\\ProcessorNameString",
-                1
-            )
-        );
-        $hw_id_md5->add(
-            _registry_info_MD5(
-"LMachine\\SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\ProductId",
-                1
-            )
-        );
-        $hardware_string = substr $hw_id_md5->digest, 0, 6;
-    }
     else {
 
         # Default case: MSWin32 was the iTunes platform that made SC Info files
         # Running Linux, BSD, or Solaris with a Windows partition mounted?
-        # If so, see if we can find a Windows registry to read (slow process)
-        # Also need various hardware stuff not in a fixed registry file
-        my $hw_id_md5 = Digest::MD5->new;
-        $hw_id_md5->add("cache-controlEthernet");
-
-        # drive serial number
-        my $drive_serial;
-        my @lines = `hdparm -I /dev/sda`;
-        my @d_txt = grep { m/Serial Number/ } @lines;
-        if ( @d_txt and $d_txt[0] =~ m/:\s*([a-zA-Z\-\s\d]+)/ ) {
-            $drive_serial = $1;
-            $hw_id_md5->add( _info_MD5( substr( $drive_serial, 0, 4 ), 0 ) );
-        }
-        else {
-            carp("Cannot find drive hda ID");
-            return;
-        }
-        $hw_id_md5->add("cache-controlEthernet");
-
-        # system bios version string
-        my $dmi = Parse::DMIDecode->new();
-        $dmi->probe;
-        my $bios_version = $dmi->smbios_version();
-        $hw_id_md5->add( _info_MD5( $bios_version, 0 ) );
-
-        # CPU name string
-        my $info     = Sys::Info->new;
-        my $cpu_info = $info->device("CPU");
-        my $cpu_name = $cpu_info->identify;
-        $hw_id_md5->add( _info_MD5( $cpu_name, 1 ) );
-
-        my $software_registry_file = _get_hive_file();
-        my $ProductId;
-        my $registry = Parse::Win32Registry->new($software_registry_file);
-        my $root_key = $registry->get_root_key;
-        my $key_name = 'Microsoft\Windows\CurrentVersion';
-        my $key      = $root_key->get_subkey($key_name);
-        for my $val ( @{ $key->get_list_of_values } ) {
-            $val->as_string =~ /^ProductId.+=\s*(.+)$/;
-            if ($1) {
-                $ProductId = $1;
-                last;
+        # If so, will need to get the registry data from a "regdata" file
+        my ( $volume, $bios, $processor, $ProductID );
+        if ( open my $infh, '<', $self->{regdata} ) {
+            my @lines = <$infh>;
+            close $infh;
+            for my $s (@lines) {
+                $s =~ s/\r|\n//g;
+                next unless $s;
+                if ( $s =~ /SystemBiosVersion\s.+SZ\s+(\S.+)$/ ) {
+                    $bios = $1;
+                }
+                elsif ( $s =~ /ProcessorNameString\s.+SZ\s+(\S.+)$/ ) {
+                    $processor = $1;
+                }
+                elsif ( $s =~ /ProductId\s.+SZ\s+(\S.+)$/ ) {
+                    $ProductID = $1;
+                }
+                elsif ( $s =~ /Volume Serial Number is\s+(\S+)$/ ) {
+                    $volume = $1;
+                }
             }
+            print "Bios: [ " , $bios , " ]\nCPU: [ ", $processor, " ]\n",
+              "Product ID: [ ", $ProductID, " ]\nVolume Serial: [ ", $volume, " ]\n"
+              if $self->{DEBUG};
+            $bios =~ s/\\0/\x00/g;
+            $volume =~ s/-//;
+            $volume = pack "L", hex $volume;
+            my $hw_id_md5 = Digest::MD5->new;
+            $hw_id_md5->add("cache-controlEthernet");
+            if ($volume) {
+                $hw_id_md5->add(
+                    _info_MD5( substr( $volume, 0, 4 ), 0 ) );
+            }
+            else {
+                carp("Cannot find volume serial number");
+                return;
+            }
+            if ($bios) {
+                $hw_id_md5->add( _info_MD5( $bios, 0 ) );
+            }
+            else {
+                carp( "Cannot find the bios string from file ",
+                    $self->{regdata_file} );
+                return;
+            }
+            if ($processor) {
+                $hw_id_md5->add( _info_MD5( $processor, 1 ) );
+            }
+            else {
+                carp( "Cannot find the CPU string from file ",
+                    $self->{regdata_file} );
+                return;
+            }
+            if ($ProductID) {
+                $hw_id_md5->add( _info_MD5( $ProductID, 1 ) );
+            }
+            else {
+                carp( "Cannot find the Windows software ProductId from file ",
+                    $self->{regdata_file} );
+                return;
+            }
+            $hardware_string = substr $hw_id_md5->digest, 0, 6;
         }
-        if ($ProductId) {
-            $hw_id_md5->add( _info_MD5( $ProductId, 1 ) );
-        }
-        else {
-            carp("Cannot find the Windows software ProductId from registry");
-            return;
-        }
-        $hardware_string = substr $hw_id_md5->digest, 0, 6;
     }
     $self->{mac} = $hardware_string;
-
+    print "hwID is: 0x", ( join '', map { sprintf '%02X', $_ }  
+      unpack "C*", $hardware_string ), " ( $hardware_string )\n" if $self->{DEBUG};
     return $hardware_string;
 }
 
@@ -547,19 +539,23 @@ sub _trim_after_itgr {
 
 sub find_veggie_for_userID {
     my ( $self, $userID ) = @_;
-    my $sd =
-      iTunes::Sid->new( file => 'SCINFOSIDD', key => 'FIND', iv => 'FIND' );
+    my $sd = iTunes::Sid->new(
+        file => 'SCINFOSIDD',
+        key  => 'FIND',
+        iv   => 'FIND',
+        regdata => $self->{regdata},
+    );
     if ( $sd->sid_version() != 0x10001 ) {
-        carp("Incomptible SC Info version");
+        carp("Incompatible SC Info version");
         return;
     }
     my $mac  = $self->mac();
     my @grup = $sd->FindAtom('grup');
-    my $own_grup;
+    my ( $own_grup, $guid_mac );
     for my $gp (@grup) {
         my $head = $gp->DirectChildren('head')   or next;
         my $guid = $head->DirectChildren('guid') or next;
-        my $guid_mac = substr $guid->NonVersionData(), 0, 6;
+        $guid_mac = substr $guid->NonVersionData(), 0, 6;
         if ( $mac eq $guid_mac ) {
             $own_grup = $gp;
             last;
@@ -581,6 +577,15 @@ sub find_veggie_for_userID {
             map { $_->MainVersion() } $own_grup->DirectChildren('user') );
         return;
       };
+    if( $auto_save_guid_and_mac_in_drms_dir and $guid_mac ) {
+        my $guid_entry_file = _drms_directory . 'guID';
+        if( ! -f $guid_entry_file ) {
+            open( my $gfh, '>', $guid_entry_file );
+            binmode $gfh;
+            print $gfh $guid_mac;
+            close $gfh;
+        }
+    }
     my $usag =
       first { $_->MainVersion == 0x2100003 } $user->DirectChildren('usag')
       or do {
@@ -609,7 +614,11 @@ sub find_veggie_for_userID {
 
 sub fetch_all_user_keys {
     my ($self) = @_;
-    my $sb = iTunes::Sid->new( file => 'SCINFOSIDB', key => 'FIND' );
+    my $sb = iTunes::Sid->new( 
+        file    => 'SCINFOSIDB', 
+        key     => 'FIND',
+        regdata => $self->{regdata},
+    );
     my @users = $sb->FindAtom('user');
     for my $usr (@users) {
         my $userID = $usr->MainVersion();
@@ -630,7 +639,8 @@ sub fetch_all_user_keys {
                 next;
             }
             my $encrypted = ( $b_key->DirectChildren('valu') )->NonVersionData;
-            print "Encrypted key $keyID is $encrypted..." if $self->{DEBUG};
+            print "Encrypted key $keyID is $encrypted..."
+              if $self->{DEBUG};
             my $vg = Crypt::AppleTwoFish->new(
                 hwID       => $self->mac(),
                 keyID      => $keyID,
@@ -650,18 +660,19 @@ sub fetch_all_user_keys {
     }
 }
 
-# Write to .drm directory (do not overwrite unless overwrite is ok )
+# Write to .drms directory (do not overwrite unless overwrite is ok )
 #   as in $sid->write_keys_to_drms_directory( overwrite_ok => 1 );
 sub write_all_user_keys_to_drms_dir {
     my ( $self, %args ) = @_;
     my $overwrite_ok = $args{overwrite_ok};
     my $drm_dir      = _drms_directory();
-    mkpath( $drm_dir );
+    mkpath($drm_dir);
     while ( my ( $userID, $keys ) = each %{ $self->{user_drm_keys} } ) {
         while ( my ( $keyID, $keyval ) = each %{$keys} ) {
             my $filename = sprintf "%s%08X.%03d", $drm_dir, $userID, $keyID;
             next if -f $filename and !$overwrite_ok;
-            print "printing key $keyval to file $filename\n" if $self->{DEBUG};
+            print "printing key $keyval to file $filename\n"
+              if $self->{DEBUG};
             open my $outf, '>', $filename or do {
                 carp(" Cannot open $filename for output: $!");
                 next;
@@ -700,11 +711,52 @@ sub write_all_user_keys_to_drms_dir {
 
 =item B<new>
 
-    my $sid = iTunes::Sid( file => "sidb", $key => 'FIND', $iv => 'FIND'  );
+        my $sid = iTunes::Sid( file => "sidb", $key => 'FIND', $iv => 'FIND'  );
+    
+        my $sid = iTunes::Sid( file => "sidb", regdata => 'filename', 
+                               iTunes_platform => 'Darwin' );
     
     Create an iTunes::Sid object. The file => $filename named argument specifies a 
     data file to be read. The $key and $iv arguments are for decryption, if needed.
+    regdata is a pathname of a file containing hardware data for key calculation.
     
+    iTunes_platform => 'Darwin'  means to look for an existing OS X installation 
+    on the mounted drives.  The default is to look for an accessible Windows 
+    partition on which to find the SC Info data.
+    
+        my $sid = iTunes::Sid( file => 'SCINFOSIDB', regdata => 'filename' );
+    
+    my $sid = iTunes::Sid( file => 'SCINFOSIDD', regdata => 'filename',
+        scinfo_direcory 
+          => '/media/windisk/ProgramData/Apple Computer/iTunes/SC Info' );
+    
+    SCINFOSIDB and SCINFOSIDD are 'magic' file names indicating we should look 
+    for the SC Info/Sc Info.sid[b|d] files and open them if found.  scinfo_directory
+    is a location to look for the SC Info.sid? files.  Remember to mount the 
+    Windows drive first, and take note of its assigned name once mounted.
+    
+        my $sid = iTunes::Sid( file => "SCINFOSIDB", REGDATAFILE => 'regdata' );
+
+    Read the Sc Info/Sc Info.sidb file using a key calculated from the regdata 
+    file created with the following Windows command file, run on the XP or 
+    Vista installation for which iTunes created its SC Info files:
+    
+    ===========================================================================
+    @echo off
+    rem file get_hwdata.cmd, run as "get_hwdata.cmd > \regdata" from Windows command prompt
+    
+    REG query "HKLM\HARDWARE\DESCRIPTION\System" /v SystemBiosVersion 
+
+    REG query "HKLM\HARDWARE\DESCRIPTION\System\CentralProcessor\0" /v ProcessorNameString 
+
+    REG query "HKLM\SOFTWARE\Microsoft\Windows\CurrentVersion" /v ProductId 
+    
+    DIR C:\Windows\PROTOCOL.INI
+
+    ===========================================================================
+
+    See the examples directory also, and see the LINUX_HOWTO in the examples.
+        
 =item B<DESTROY>
 
     Not to be called directly-- this cleans up circular references, if any.
@@ -822,6 +874,56 @@ sub write_all_user_keys_to_drms_dir {
 
 =back
 
+=head1 LINUX HOWTO 
+
+=over 4
+
+The following is intended for fair use of purchased music by the purchaser, not
+for piracy. In order to format the FairPlay keys used by iTunes for use by a 
+music player such as VLC under Linux, the keys must be pulled from the 
+SC Info.sidb file, translated using information in the SC Info. sidd file, and
+then written to a directory where VLC will read the keys for its use.  Note that
+such use does B<not> remove the DRM from the purchased music files themselves, 
+but makes them far more useable under Linux, without resorting to Wine.
+
+Procedure:
+
+1. Set up a machine for dual boot under the iTunes installation OS (OS X or 
+Windows).  One partition is to boot OS X or Windows, another Linux. Make sure 
+that your Darwin or NTFS partition is mounted at least for reading!
+
+2A.  (iTunes for Windows) Run the file ./examples/get_hwdata.cmd in the 
+     iTunes::Sid /examples directory in this distribution and redirect to a 
+     file given as the command's argument, as in
+
+     c:> get_hwdata.cmd c:\regdata
+
+    Then reboot to Linux and mount the windows partion (it should be attached 
+    to /media/disk1/ or /media/disk2, etc).  
+    
+    Run the file ./examples/get_keys_for_vlc.pl (the file pathname on line 10 
+    may need to be changed to be directed to the output file of the Windows 
+    command file above).
+
+2B. (iTunes for OS X) Dual boot may "just work" since the key is derived 
+    from the computer's LAN card MAC address.  If attaching to the OS X 
+    machine over a network share, you may get the mac address of the source 
+    machine by running ./examples/print_mac.pl and redirecting this to a file, 
+    then copying that file to your home directory into a subdirectory called 
+    '.drms' and a file called 'mac' in that directory. iTunes::Sid will look 
+    for this file, and use it for the mac adddress if it finds that file.  
+
+3.  Run the ./examples/get_keys_for_vlc.pl perl script to allow any DRM-laden
+    iTMS purchases to play under VLC or other compatible media players.
+
+    If the above fails, you may be able to run the above under OS X or Windows, 
+    and then copy the drms directory and its keys to your linux home directory,
+    renaming 'drms' to '.drms' and then run VLC to play music as usual.  (The same 
+    must be done if the keys are generated under the /root account-- copy the 
+    /root/.drms directory to your home directory).
+
+=back
+
 =head1 SEE ALSO
 
 =over 4
@@ -836,7 +938,11 @@ sub write_all_user_keys_to_drms_dir {
 
 =over 4
 
-Lots.  Windows Vista dual boot compatibility seems flawed.
+Initial setup almost requires a dual boot machine.  
+
+Searches for SC Info files are subject to file finding errors. 
+
+No doubt many others.
 
 =back
 
@@ -903,8 +1009,8 @@ Questions, feature requests and bug reports should go to <wherrera@skylightview.
     Solaris, and Unix systems to achieve interoperability with iTunes and the 
     iTunes Music Store, they ARE legal under the DMCA. We are somewhat 
     concerned about the word "solely" in the text above, since this package 
-    works under OS X and Windows as well as Linux.  For that reason, please 
-    also see the COPYRIGHT below.
+    works under OS X and Windows as well as Linux.  For "solely" that reason, 
+    please also see the COPYRIGHT below.
 
 
 =head1 COPYRIGHT
