@@ -40,7 +40,7 @@ our %sid_container_atom_types = (
 our %sid_noncontainer_atom_types = (
     vers => 1,    # version data: 12 bytes 0x000100000000 then 4 bytes 0x01010
     guid => 1,    # iTMS guid, data: 12 bytes 0x000100000000 then a 6-byte value
-    valu => 1,    # 36 bytes, data: 12 bytes of 0x00010000 then a 16-byte value
+    valu => 1,    # 36 bytes, data: 12 bytes of 0x00010000 then variable data
     type => 1,    # 24 bytes, usually data is a redundant 0x000100000000
     sign => 1,    # signature, 148 bytes, 0x00010000000 then a 128-byte value
     itgr => 1,    # the last atom in the sid, unclear use; padding to 2**5*n?
